@@ -28,12 +28,35 @@ or implied, of the FreeBSD Project.
 
 import unittest
 from xml_models import *
+from xml_models.xml_models_stub import stub
 import xml_models.xpath_twister as xpath
 import rest_client
 from mock import patch_object
 from StringIO import StringIO
 from stubserver import StubServer
 
+class Address(Model):
+    number = IntField(xpath='/address/number')
+    street = CharField(xpath='/address/street')
+    city = CharField(xpath='/address/city')
+    foobars = Collection(CharField, xpath='/address/foobar')
+
+    finders = { (number,): "http://address/number/%s",
+                (number, street): "http://address/number/%s/street/%s",
+                (city,): "http://localhost:8998/address/%s"
+              }
+
+class MyModel(Model):
+    muppet_name = CharField(xpath='/root/kiddie/value')
+    muppet_type = CharField(xpath='/root/kiddie/type', default='frog')
+    muppet_names = Collection(CharField, xpath='/root/kiddie/value')
+    muppet_ages = Collection(IntField, xpath='/root/kiddie/age')
+    muppet_addresses = Collection(Address, xpath='/root/kiddie/address', order_by='number')
+
+    finders = { 
+                (muppet_name,): "http://foo.com/muppets/%s"
+              }
+
 class XmlModelsTest(unittest.TestCase):
     
     def test_char_field_returns_xpathed_value_for_the_node_passed_in(self):
@@ -297,6 +320,33 @@ class XmlModelsTest(unittest.TestCase):
         mock_get.return_value = t()
         qry = Simple.objects.filter(field1="baz")
         self.assertEquals(2, len(qry))
+        
+    @stub(MyModel)
+    def test_stub_allows_stubbing_return_values_for_queries(self):
+        MyModel.stub().get(muppet_name='Kermit').returns(muppet_name='Kermit', muppet_type='toad', muppet_names=['Trevor', 'Kyle'])
+        result = MyModel.objects.get(muppet_name='Kermit')
+        self.assertEquals('toad', result.muppet_type)
+        
+    @stub(MyModel)
+    def test_stub_allows_stubbing_filter_requests(self):
+        MyModel.stub().filter(muppet_name='Kermit').returns(dict(muppet_name='Kermit', muppet_type='toad', muppet_names=['Trevor', 'Kyle']))
+        result = MyModel.objects.filter(muppet_name='Kermit')
+        self.assertEquals(1, len(result))
+        self.assertEquals('toad',list(result)[0].muppet_type)
+    
+    @stub(MyModel)
+    def test_stub_allows_stubbing_filter_custom_requests(self):
+        MyModel.stub().filter_custom('http://anyurl.com').returns(dict(muppet_name='Kermit', muppet_type='toad', muppet_names=['Trevor', 'Kyle']))
+        result = MyModel.objects.filter_custom('http://anyurl.com')
+        self.assertEquals(1, len(result))
+        self.assertEquals('toad',list(result)[0].muppet_type)
+    
+    def test_headers_field_specified_on_model_is_added_to_the_query_manager(self):
+        self.assertTrue(Simple.objects.headers != None)
+        self.assertEquals('user1', Simple.objects.headers['user'])
+        query = Simple.objects.filter(field1="Rhubarb")
+        self.assertTrue(query.headers != None)
+        self.assertEquals('pwd1', query.headers['password'])
     
 class FunctionalTest(unittest.TestCase):
     def setUp(self):
@@ -311,28 +361,7 @@ class FunctionalTest(unittest.TestCase):
         self.server.expect(method="GET", url="/address/\w+$").and_return(mime_type="text/xml", content="<address><number>12</number><street>Early Drive</street><city>Calgary</city></address>")
         address = Address.objects.get(city="Calgary")
         self.assertEquals("Early Drive", address.street)
-        
-class Address(Model):
-    number = IntField(xpath='/address/number')
-    street = CharField(xpath='/address/street')
-    city = CharField(xpath='/address/city')
-    foobars = Collection(CharField, xpath='/address/foobar')
 
-    finders = { (number,): "http://address/number/%s",
-                (number, street): "http://address/number/%s/street/%s",
-                (city,): "http://localhost:8998/address/%s"
-              }
-
-class MyModel(Model):
-    muppet_name = CharField(xpath='/root/kiddie/value')
-    muppet_type = CharField(xpath='/root/kiddie/type', default='frog')
-    muppet_names = Collection(CharField, xpath='/root/kiddie/value')
-    muppet_ages = Collection(IntField, xpath='/root/kiddie/age')
-    muppet_addresses = Collection(Address, xpath='/root/kiddie/address', order_by='number')
-
-    finders = { 
-                (muppet_name,): "http://foo.com/muppets/%s"
-              }
 
 class MyValidatingModel(Model):
     muppet_name = CharField(xpath='/root/kiddie/value')
@@ -360,6 +389,7 @@ class Simple(Model):
     finders = {
                (field1,): "http://foo.com/simple/%s"
               }
+    headers = {'user': 'user1', 'password': 'pwd1'}
 class SimpleWithoutFinder(Model):
     field1 = CharField(xpath='/root/field1')
 
